@@ -1,40 +1,41 @@
-import re
-from itertools import cycle
-from time import sleep
-from typing import Any, Dict, List, Tuple, Optional, Set
 import datetime
-import time
 import logging
+import re
 import threading
-import dateutil.parser
+import time
+from itertools import cycle
 from sys import platform
+from time import sleep
+from typing import Any, Dict, List, Optional, Set, Tuple
 
-from moto.core import BaseBackend, BackendDict, BaseModel, CloudFormationModel
-from moto.iam.models import iam_backends, IAMBackend
-from moto.ec2.models import ec2_backends, EC2Backend
-from moto.ec2.models.instances import Instance
-from moto.ecs.models import ecs_backends, EC2ContainerServiceBackend
-from moto.logs.models import logs_backends, LogsBackend
-from moto.utilities.tagging_service import TaggingService
+import dateutil.parser
 
-from .exceptions import InvalidParameterValueException, ClientException, ValidationError
-from .utils import (
-    make_arn_for_compute_env,
-    make_arn_for_job_queue,
-    make_arn_for_job,
-    make_arn_for_task_def,
-    lowercase_first_key,
-)
-from moto.ec2.exceptions import InvalidSubnetIdError
-from moto.ec2.models.instance_types import INSTANCE_TYPES as EC2_INSTANCE_TYPES
-from moto.ec2.models.instance_types import INSTANCE_FAMILIES as EC2_INSTANCE_FAMILIES
-from moto.iam.exceptions import IAMNotFoundException
+from moto import settings
+from moto.core import BackendDict, BaseBackend, BaseModel, CloudFormationModel
 from moto.core.utils import unix_time_millis
-from moto.moto_api import state_manager
+from moto.ec2.exceptions import InvalidSubnetIdError
+from moto.ec2.models import EC2Backend, ec2_backends
+from moto.ec2.models.instance_types import INSTANCE_FAMILIES as EC2_INSTANCE_FAMILIES
+from moto.ec2.models.instance_types import INSTANCE_TYPES as EC2_INSTANCE_TYPES
+from moto.ec2.models.instances import Instance
+from moto.ecs.models import EC2ContainerServiceBackend, ecs_backends
+from moto.iam.exceptions import IAMNotFoundException
+from moto.iam.models import IAMBackend, iam_backends
+from moto.logs.models import LogsBackend, logs_backends
 from moto.moto_api._internal import mock_random
 from moto.moto_api._internal.managed_state_model import ManagedState
 from moto.utilities.docker_utilities import DockerModel
-from moto import settings
+from moto.utilities.tagging_service import TaggingService
+
+from .exceptions import ClientException, InvalidParameterValueException, ValidationError
+from .utils import (
+    JobStatus,
+    lowercase_first_key,
+    make_arn_for_compute_env,
+    make_arn_for_job,
+    make_arn_for_job_queue,
+    make_arn_for_task_def,
+)
 
 logger = logging.getLogger(__name__)
 COMPUTE_ENVIRONMENT_NAME_REGEX = re.compile(
@@ -112,16 +113,13 @@ class ComputeEnvironment(CloudFormationModel):
         backend = batch_backends[account_id][region_name]
         properties = cloudformation_json["Properties"]
 
-        env = backend.create_compute_environment(
+        return backend.create_compute_environment(
             resource_name,
             properties["Type"],
             properties.get("State", "ENABLED"),
             lowercase_first_key(properties["ComputeResources"]),
             properties["ServiceRole"],
         )
-        arn = env[1]
-
-        return backend.get_compute_environment_by_arn(arn)
 
 
 class JobQueue(CloudFormationModel):
@@ -207,16 +205,13 @@ class JobQueue(CloudFormationModel):
             for dict_item in properties["ComputeEnvironmentOrder"]
         ]
 
-        queue = backend.create_job_queue(
+        return backend.create_job_queue(
             queue_name=resource_name,
             priority=properties["Priority"],
             state=properties.get("State", "ENABLED"),
             compute_env_order=compute_envs,
-            schedule_policy={},
+            schedule_policy=None,
         )
-        arn = queue[1]
-
-        return backend.get_job_queue_by_arn(arn)
 
 
 class JobDefinition(CloudFormationModel):
@@ -445,29 +440,26 @@ class JobDefinition(CloudFormationModel):
     ) -> "JobDefinition":
         backend = batch_backends[account_id][region_name]
         properties = cloudformation_json["Properties"]
-        res = backend.register_job_definition(
+        return backend.register_job_definition(
             def_name=resource_name,
             parameters=lowercase_first_key(properties.get("Parameters", {})),
             _type="container",
             tags=lowercase_first_key(properties.get("Tags", {})),
             retry_strategy=lowercase_first_key(properties["RetryStrategy"]),
             container_properties=(
-                lowercase_first_key(properties["ContainerProperties"])
+                lowercase_first_key(properties["ContainerProperties"])  # type: ignore[arg-type]
                 if "ContainerProperties" in properties
                 else None
             ),
             node_properties=(
-                lowercase_first_key(properties["NodeProperties"])
+                lowercase_first_key(properties["NodeProperties"])  # type: ignore[arg-type]
                 if "NodeProperties" in properties
                 else None
             ),
             timeout=lowercase_first_key(properties.get("timeout", {})),
-            platform_capabilities=None,
-            propagate_tags=None,
+            platform_capabilities=None,  # type: ignore[arg-type]
+            propagate_tags=None,  # type: ignore[arg-type]
         )
-        arn = res[1]
-
-        return backend.get_job_definition_by_arn(arn)
 
 
 class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
@@ -481,22 +473,19 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
         depends_on: Optional[List[Dict[str, str]]],
         all_jobs: Dict[str, "Job"],
         timeout: Optional[Dict[str, int]],
+        array_properties: Dict[str, Any],
+        provided_job_id: Optional[str] = None,
     ):
         threading.Thread.__init__(self)
         DockerModel.__init__(self)
         ManagedState.__init__(
             self,
             "batch::job",
-            [
-                ("SUBMITTED", "PENDING"),
-                ("PENDING", "RUNNABLE"),
-                ("RUNNABLE", "STARTING"),
-                ("STARTING", "RUNNING"),
-            ],
+            JobStatus.status_transitions(),
         )
 
         self.job_name = name
-        self.job_id = str(mock_random.uuid4())
+        self.job_id = provided_job_id or str(mock_random.uuid4())
         self.job_definition = job_def
         self.container_overrides: Dict[str, Any] = container_overrides or {}
         self.job_queue = job_queue
@@ -509,6 +498,7 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
         self.depends_on = depends_on
         self.timeout = timeout
         self.all_jobs = all_jobs
+        self.array_properties: Dict[str, Any] = array_properties
 
         self.arn = make_arn_for_job(
             job_def.backend.account_id, self.job_id, job_def._region
@@ -518,6 +508,7 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
         self.exit_code: Optional[int] = None
 
         self.daemon = True
+
         self.name = "MOTO-BATCH-" + self.job_id
 
         self._log_backend = log_backend
@@ -527,6 +518,7 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
 
         self.attempts: List[Dict[str, Any]] = []
         self.latest_attempt: Optional[Dict[str, Any]] = None
+        self._child_jobs: Optional[List[Job]] = None
 
     def describe_short(self) -> Dict[str, Any]:
         result = {
@@ -539,8 +531,9 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
         }
         if self.job_stopped_reason is not None:
             result["statusReason"] = self.job_stopped_reason
-        if result["status"] not in ["SUBMITTED", "PENDING", "RUNNABLE", "STARTING"]:
-            result["startedAt"] = datetime2int_milliseconds(self.job_started_at)
+        if self.status is not None:
+            if JobStatus.is_job_already_started(self.status):
+                result["startedAt"] = datetime2int_milliseconds(self.job_started_at)
         if self.job_stopped:
             result["stoppedAt"] = datetime2int_milliseconds(self.job_stopped_at)
             if self.exit_code is not None:
@@ -563,6 +556,26 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
         if self.timeout:
             result["timeout"] = self.timeout
         result["attempts"] = self.attempts
+        if self._child_jobs:
+            child_statuses = {
+                "STARTING": 0,
+                "FAILED": 0,
+                "RUNNING": 0,
+                "SUCCEEDED": 0,
+                "RUNNABLE": 0,
+                "SUBMITTED": 0,
+                "PENDING": 0,
+            }
+            for child_job in self._child_jobs:
+                if child_job.status is not None:
+                    child_statuses[child_job.status] += 1
+            result["arrayProperties"] = {
+                "statusSummary": child_statuses,
+                "size": len(self._child_jobs),
+            }
+            if len(self._child_jobs) == child_statuses["SUCCEEDED"]:
+                self.status = "SUCCEEDED"
+                result["status"] = self.status
         return result
 
     def _container_details(self) -> Dict[str, Any]:
@@ -638,7 +651,7 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
             containers: List[docker.models.containers.Container] = []
 
             self.advance()
-            while self.status == "SUBMITTED":
+            while self.status == JobStatus.SUBMITTED:
                 # Wait until we've moved onto state 'PENDING'
                 sleep(0.5)
 
@@ -678,7 +691,7 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
                             )
                             for m in self._get_container_property("mountPoints", [])
                         ],
-                        "name": f"{self.job_name}-{self.job_id}",
+                        "name": f"{self.job_name}-{self.job_id.replace(':', '-')}",
                     }
                 )
             else:
@@ -730,7 +743,7 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
                     )
 
             self.advance()
-            while self.status == "PENDING":
+            while self.status == JobStatus.PENDING:
                 # Wait until the state is no longer pending, but 'RUNNABLE'
                 sleep(0.5)
             # TODO setup ecs container instance
@@ -765,12 +778,12 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
 
             log_config = docker.types.LogConfig(type=docker.types.LogConfig.types.JSON)
             self.advance()
-            while self.status == "RUNNABLE":
+            while self.status == JobStatus.RUNNABLE:
                 # Wait until the state is no longer runnable, but 'STARTING'
                 sleep(0.5)
 
             self.advance()
-            while self.status == "STARTING":
+            while self.status == JobStatus.STARTING:
                 # Wait until the state is no longer runnable, but 'RUNNING'
                 sleep(0.5)
 
@@ -898,7 +911,7 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
         # The describe-method needs them immediately when status is set
         self.job_stopped = True
         self.job_stopped_at = datetime.datetime.now()
-        self.status = "SUCCEEDED" if success else "FAILED"
+        self.status = JobStatus.SUCCEEDED if success else JobStatus.FAILED
         self._stop_attempt()
 
     def _start_attempt(self) -> None:
@@ -934,9 +947,9 @@ class Job(threading.Thread, BaseModel, DockerModel, ManagedState):
             for dependent_id in dependent_ids:
                 if dependent_id in self.all_jobs:
                     dependent_job = self.all_jobs[dependent_id]
-                    if dependent_job.status == "SUCCEEDED":
+                    if dependent_job.status == JobStatus.SUCCEEDED:
                         successful_dependencies.add(dependent_id)
-                    if dependent_job.status == "FAILED":
+                    if dependent_job.status == JobStatus.FAILED:
                         logger.error(
                             f"Terminating job {self.name} due to failed dependency {dependent_job.name}"
                         )
@@ -1000,10 +1013,6 @@ class BatchBackend(BaseBackend):
         self._jobs: Dict[str, Job] = {}
         self._scheduling_policies: Dict[str, SchedulingPolicy] = {}
 
-        state_manager.register_default_transition(
-            "batch::job", transition={"progression": "manual", "times": 1}
-        )
-
     @property
     def iam_backend(self) -> IAMBackend:
         """
@@ -1038,10 +1047,11 @@ class BatchBackend(BaseBackend):
 
     def reset(self) -> None:
         for job in self._jobs.values():
-            if job.status not in ("FAILED", "SUCCEEDED"):
+            if job.status not in (JobStatus.FAILED, JobStatus.SUCCEEDED):
                 job.stop = True
                 # Try to join
-                job.join(0.2)
+                if job.is_alive():
+                    job.join(0.2)
 
         super().reset()
 
@@ -1196,7 +1206,7 @@ class BatchBackend(BaseBackend):
         state: str,
         compute_resources: Dict[str, Any],
         service_role: str,
-    ) -> Tuple[str, str]:
+    ) -> ComputeEnvironment:
         # Validate
         if COMPUTE_ENVIRONMENT_NAME_REGEX.match(compute_environment_name) is None:
             raise InvalidParameterValueException(
@@ -1281,7 +1291,7 @@ class BatchBackend(BaseBackend):
         ecs_cluster = self.ecs_backend.create_cluster(cluster_name)
         new_comp_env.set_ecs(ecs_cluster.arn, cluster_name)
 
-        return compute_environment_name, new_comp_env.arn
+        return new_comp_env
 
     def _validate_compute_resources(self, cr: Dict[str, Any]) -> None:
         """
@@ -1481,7 +1491,7 @@ class BatchBackend(BaseBackend):
         state: str,
         compute_env_order: List[Dict[str, str]],
         tags: Optional[Dict[str, str]] = None,
-    ) -> Tuple[str, str]:
+    ) -> JobQueue:
         for variable, var_name in (
             (queue_name, "jobQueueName"),
             (priority, "priority"),
@@ -1527,7 +1537,7 @@ class BatchBackend(BaseBackend):
         )
         self._job_queues[queue.arn] = queue
 
-        return queue_name, queue.arn
+        return queue
 
     def describe_job_queues(
         self, job_queues: Optional[List[str]] = None
@@ -1621,7 +1631,7 @@ class BatchBackend(BaseBackend):
         timeout: Dict[str, int],
         platform_capabilities: List[str],
         propagate_tags: bool,
-    ) -> Tuple[str, str, int]:
+    ) -> JobDefinition:
         if def_name is None:
             raise ClientException("jobDefinitionName must be provided")
 
@@ -1660,7 +1670,7 @@ class BatchBackend(BaseBackend):
 
         self._job_definitions[job_def.arn] = job_def
 
-        return def_name, job_def.arn, job_def.revision
+        return job_def
 
     def deregister_job_definition(self, def_name: str) -> None:
         job_def = self.get_job_definition_by_arn(def_name)
@@ -1707,10 +1717,11 @@ class BatchBackend(BaseBackend):
         job_name: str,
         job_def_id: str,
         job_queue: str,
+        array_properties: Dict[str, int],
         depends_on: Optional[List[Dict[str, str]]] = None,
         container_overrides: Optional[Dict[str, Any]] = None,
         timeout: Optional[Dict[str, int]] = None,
-    ) -> Tuple[str, str]:
+    ) -> Tuple[str, str, str]:
         """
         Parameters RetryStrategy and Parameters are not yet implemented.
         """
@@ -1735,13 +1746,37 @@ class BatchBackend(BaseBackend):
             depends_on=depends_on,
             all_jobs=self._jobs,
             timeout=timeout,
+            array_properties=array_properties or {},
         )
         self._jobs[job.job_id] = job
 
-        # Here comes the fun
-        job.start()
+        if "size" in array_properties:
+            child_jobs = []
+            for array_index in range(array_properties["size"]):
+                provided_job_id = f"{job.job_id}:{array_index}"
+                child_job = Job(
+                    job_name,
+                    job_def,
+                    queue,
+                    log_backend=self.logs_backend,
+                    container_overrides=container_overrides,
+                    depends_on=depends_on,
+                    all_jobs=self._jobs,
+                    timeout=timeout,
+                    array_properties={"statusSummary": {}, "index": array_index},
+                    provided_job_id=provided_job_id,
+                )
+                child_jobs.append(child_job)
+                self._jobs[child_job.job_id] = child_job
+                child_job.start()
 
-        return job_name, job.job_id
+            # The 'parent' job doesn't need to be executed
+            # it just needs to keep track of it's children
+            job._child_jobs = child_jobs
+        else:
+            # Here comes the fun
+            job.start()
+        return job_name, job.job_id, job.arn
 
     def describe_jobs(self, jobs: Optional[List[str]]) -> List[Dict[str, Any]]:
         job_filter = set()
@@ -1776,15 +1811,7 @@ class BatchBackend(BaseBackend):
         if job_queue is None:
             raise ClientException(f"Job queue {job_queue_name} does not exist")
 
-        if job_status is not None and job_status not in (
-            "SUBMITTED",
-            "PENDING",
-            "RUNNABLE",
-            "STARTING",
-            "RUNNING",
-            "SUCCEEDED",
-            "FAILED",
-        ):
+        if job_status is not None and job_status not in JobStatus.job_statuses():
             raise ClientException(
                 "Job status is not one of SUBMITTED | PENDING | RUNNABLE | STARTING | RUNNING | SUCCEEDED | FAILED"
             )
@@ -1821,7 +1848,9 @@ class BatchBackend(BaseBackend):
 
         job = self.get_job_by_id(job_id)
         if job is not None:
-            if job.status in ["SUBMITTED", "PENDING", "RUNNABLE"]:
+            if job.status is None:
+                return
+            if JobStatus.is_job_before_starting(job.status):
                 job.terminate(reason)
             # No-Op for jobs that have already started - user has to explicitly terminate those
 
