@@ -1,25 +1,25 @@
 from moto.core.responses import BaseResponse
 
-from .models import dynamodbstreams_backends
+from .models import DynamoDBStreamsBackend, dynamodbstreams_backends
 
 
 class DynamoDBStreamsHandler(BaseResponse):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__(service_name="dynamodb-streams")
 
     @property
-    def backend(self):
+    def backend(self) -> DynamoDBStreamsBackend:
         return dynamodbstreams_backends[self.current_account][self.region]
 
-    def describe_stream(self):
+    def describe_stream(self) -> str:
         arn = self._get_param("StreamArn")
         return self.backend.describe_stream(arn)
 
-    def list_streams(self):
+    def list_streams(self) -> str:
         table_name = self._get_param("TableName")
         return self.backend.list_streams(table_name)
 
-    def get_shard_iterator(self):
+    def get_shard_iterator(self) -> str:
         arn = self._get_param("StreamArn")
         shard_id = self._get_param("ShardId")
         shard_iterator_type = self._get_param("ShardIteratorType")
@@ -32,7 +32,7 @@ class DynamoDBStreamsHandler(BaseResponse):
             arn, shard_id, shard_iterator_type, sequence_number
         )
 
-    def get_records(self):
+    def get_records(self) -> str:
         arn = self._get_param("ShardIterator")
         limit = self._get_param("Limit")
         if limit is None:
