@@ -1,23 +1,24 @@
 from moto.core.utils import camelcase_to_underscores
 from moto.ec2.utils import add_tag_specification
+
 from ._base_response import EC2BaseResponse
 
 
 class VPCs(EC2BaseResponse):
-    def _get_doc_date(self):
+    def _get_doc_date(self) -> str:
         return (
             "2013-10-15"
             if "Boto/" in self.headers.get("user-agent", "")
             else "2016-11-15"
         )
 
-    def create_default_vpc(self):
+    def create_default_vpc(self) -> str:
         vpc = self.ec2_backend.create_default_vpc()
         doc_date = self._get_doc_date()
         template = self.response_template(CREATE_VPC_RESPONSE)
         return template.render(vpc=vpc, doc_date=doc_date)
 
-    def create_vpc(self):
+    def create_vpc(self) -> str:
         cidr_block = self._get_param("CidrBlock")
         tags = self._get_multi_param("TagSpecification")
         instance_tenancy = self._get_param("InstanceTenancy", if_none="default")
@@ -43,13 +44,13 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(CREATE_VPC_RESPONSE)
         return template.render(vpc=vpc, doc_date=doc_date)
 
-    def delete_vpc(self):
+    def delete_vpc(self) -> str:
         vpc_id = self._get_param("VpcId")
         vpc = self.ec2_backend.delete_vpc(vpc_id)
         template = self.response_template(DELETE_VPC_RESPONSE)
         return template.render(vpc=vpc)
 
-    def describe_vpcs(self):
+    def describe_vpcs(self) -> str:
         self.error_on_dryrun()
         vpc_ids = self._get_multi_param("VpcId")
         filters = self._filters_from_querystring()
@@ -62,14 +63,13 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(DESCRIBE_VPCS_RESPONSE)
         return template.render(vpcs=vpcs, doc_date=doc_date, region=self.region)
 
-    def modify_vpc_tenancy(self):
+    def modify_vpc_tenancy(self) -> str:
         vpc_id = self._get_param("VpcId")
         tenancy = self._get_param("InstanceTenancy")
-        value = self.ec2_backend.modify_vpc_tenancy(vpc_id, tenancy)
-        template = self.response_template(MODIFY_VPC_TENANCY_RESPONSE)
-        return template.render(value=value)
+        self.ec2_backend.modify_vpc_tenancy(vpc_id, tenancy)
+        return self.response_template(MODIFY_VPC_TENANCY_RESPONSE).render()
 
-    def describe_vpc_attribute(self):
+    def describe_vpc_attribute(self) -> str:
         vpc_id = self._get_param("VpcId")
         attribute = self._get_param("Attribute")
         attr_name = camelcase_to_underscores(attribute)
@@ -77,7 +77,7 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(DESCRIBE_VPC_ATTRIBUTE_RESPONSE)
         return template.render(vpc_id=vpc_id, attribute=attribute, value=value)
 
-    def describe_vpc_classic_link_dns_support(self):
+    def describe_vpc_classic_link_dns_support(self) -> str:
         vpc_ids = self._get_multi_param("VpcIds")
         filters = self._filters_from_querystring()
         vpcs = self.ec2_backend.describe_vpcs(vpc_ids=vpc_ids, filters=filters)
@@ -87,7 +87,7 @@ class VPCs(EC2BaseResponse):
         )
         return template.render(vpcs=vpcs, doc_date=doc_date)
 
-    def enable_vpc_classic_link_dns_support(self):
+    def enable_vpc_classic_link_dns_support(self) -> str:
         vpc_id = self._get_param("VpcId")
         classic_link_dns_supported = (
             self.ec2_backend.enable_vpc_classic_link_dns_support(vpc_id=vpc_id)
@@ -98,7 +98,7 @@ class VPCs(EC2BaseResponse):
             classic_link_dns_supported=classic_link_dns_supported, doc_date=doc_date
         )
 
-    def disable_vpc_classic_link_dns_support(self):
+    def disable_vpc_classic_link_dns_support(self) -> str:
         vpc_id = self._get_param("VpcId")
         classic_link_dns_supported = (
             self.ec2_backend.disable_vpc_classic_link_dns_support(vpc_id=vpc_id)
@@ -109,7 +109,7 @@ class VPCs(EC2BaseResponse):
             classic_link_dns_supported=classic_link_dns_supported, doc_date=doc_date
         )
 
-    def describe_vpc_classic_link(self):
+    def describe_vpc_classic_link(self) -> str:
         vpc_ids = self._get_multi_param("VpcId")
         filters = self._filters_from_querystring()
         vpcs = self.ec2_backend.describe_vpcs(vpc_ids=vpc_ids, filters=filters)
@@ -117,7 +117,7 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(DESCRIBE_VPC_CLASSIC_LINK_RESPONSE)
         return template.render(vpcs=vpcs, doc_date=doc_date)
 
-    def enable_vpc_classic_link(self):
+    def enable_vpc_classic_link(self) -> str:
         vpc_id = self._get_param("VpcId")
         classic_link_enabled = self.ec2_backend.enable_vpc_classic_link(vpc_id=vpc_id)
         doc_date = self._get_doc_date()
@@ -126,7 +126,7 @@ class VPCs(EC2BaseResponse):
             classic_link_enabled=classic_link_enabled, doc_date=doc_date
         )
 
-    def disable_vpc_classic_link(self):
+    def disable_vpc_classic_link(self) -> str:
         vpc_id = self._get_param("VpcId")
         classic_link_enabled = self.ec2_backend.disable_vpc_classic_link(vpc_id=vpc_id)
         doc_date = self._get_doc_date()
@@ -135,18 +135,21 @@ class VPCs(EC2BaseResponse):
             classic_link_enabled=classic_link_enabled, doc_date=doc_date
         )
 
-    def modify_vpc_attribute(self):
+    def modify_vpc_attribute(self) -> str:
         vpc_id = self._get_param("VpcId")
-
-        for attribute in ("EnableDnsSupport", "EnableDnsHostnames"):
-            if self.querystring.get("%s.Value" % attribute):
+        for attribute in (
+            "EnableDnsSupport",
+            "EnableDnsHostnames",
+            "EnableNetworkAddressUsageMetrics",
+        ):
+            if self.querystring.get(f"{attribute}.Value"):
                 attr_name = camelcase_to_underscores(attribute)
-                attr_value = self.querystring.get("%s.Value" % attribute)[0]
+                attr_value = self.querystring[f"{attribute}.Value"][0]
                 self.ec2_backend.modify_vpc_attribute(vpc_id, attr_name, attr_value)
                 return MODIFY_VPC_ATTRIBUTE_RESPONSE
-        return None
+        return ""
 
-    def associate_vpc_cidr_block(self):
+    def associate_vpc_cidr_block(self) -> str:
         vpc_id = self._get_param("VpcId")
         amazon_provided_ipv6_cidr_blocks = self._get_param(
             "AmazonProvidedIpv6CidrBlock"
@@ -173,7 +176,7 @@ class VPCs(EC2BaseResponse):
             cidr_block_state="associating",
         )
 
-    def disassociate_vpc_cidr_block(self):
+    def disassociate_vpc_cidr_block(self) -> str:
         association_id = self._get_param("AssociationId")
         value = self.ec2_backend.disassociate_vpc_cidr_block(association_id)
         if "::" in value.get("cidr_block", ""):
@@ -188,7 +191,7 @@ class VPCs(EC2BaseResponse):
             cidr_block_state="disassociating",
         )
 
-    def create_vpc_endpoint(self):
+    def create_vpc_endpoint(self) -> str:
         vpc_id = self._get_param("VpcId")
         service_name = self._get_param("ServiceName")
         route_table_ids = self._get_multi_param("RouteTableId")
@@ -196,11 +199,10 @@ class VPCs(EC2BaseResponse):
         endpoint_type = self._get_param("VpcEndpointType")
         policy_document = self._get_param("PolicyDocument")
         client_token = self._get_param("ClientToken")
-        tags = self._get_multi_param("TagSpecification")
         private_dns_enabled = self._get_bool_param("PrivateDnsEnabled", if_none=True)
         security_group_ids = self._get_multi_param("SecurityGroupId")
 
-        tags = add_tag_specification(tags)
+        tags = add_tag_specification(self._get_multi_param("TagSpecification"))
         vpc_end_point = self.ec2_backend.create_vpc_endpoint(
             vpc_id=vpc_id,
             service_name=service_name,
@@ -216,7 +218,7 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(CREATE_VPC_END_POINT)
         return template.render(vpc_end_point=vpc_end_point)
 
-    def modify_vpc_endpoint(self):
+    def modify_vpc_endpoint(self) -> str:
         vpc_id = self._get_param("VpcEndpointId")
         add_subnets = self._get_multi_param("AddSubnetId")
         add_route_tables = self._get_multi_param("AddRouteTableId")
@@ -232,9 +234,8 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(MODIFY_VPC_END_POINT)
         return template.render()
 
-    def describe_vpc_endpoint_services(self):
+    def describe_vpc_endpoint_services(self) -> str:
         vpc_end_point_services = self.ec2_backend.describe_vpc_endpoint_services(
-            dry_run=self._get_bool_param("DryRun"),
             service_names=self._get_multi_param("ServiceName"),
             filters=self._get_multi_param("Filter"),
             max_results=self._get_int_param("MaxResults"),
@@ -244,7 +245,7 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(DESCRIBE_VPC_ENDPOINT_SERVICES_RESPONSE)
         return template.render(vpc_end_points=vpc_end_point_services)
 
-    def describe_vpc_endpoints(self):
+    def describe_vpc_endpoints(self) -> str:
         vpc_end_points_ids = self._get_multi_param("VpcEndpointId")
         filters = self._filters_from_querystring()
         vpc_end_points = self.ec2_backend.describe_vpc_endpoints(
@@ -255,22 +256,18 @@ class VPCs(EC2BaseResponse):
             vpc_end_points=vpc_end_points, account_id=self.current_account
         )
 
-    def delete_vpc_endpoints(self):
+    def delete_vpc_endpoints(self) -> str:
         vpc_end_points_ids = self._get_multi_param("VpcEndpointId")
-        response = self.ec2_backend.delete_vpc_endpoints(vpce_ids=vpc_end_points_ids)
-        template = self.response_template(DELETE_VPC_ENDPOINT_RESPONSE)
-        return template.render(response=response)
+        self.ec2_backend.delete_vpc_endpoints(vpce_ids=vpc_end_points_ids)
+        return self.response_template(DELETE_VPC_ENDPOINT_RESPONSE).render()
 
-    def create_managed_prefix_list(self):
+    def create_managed_prefix_list(self) -> str:
         address_family = self._get_param("AddressFamily")
         max_entries = self._get_param("MaxEntries")
         prefix_list_name = self._get_param("PrefixListName")
         entry = self._get_multi_param("Entry")
 
-        tags = self._get_multi_param("TagSpecification")
-        tags = tags[0] if isinstance(tags, list) and len(tags) == 1 else tags
-        tags = (tags or {}).get("Tag", [])
-        tags = {t["Key"]: t["Value"] for t in tags}
+        tags = self._parse_tag_specification().get("prefix-list", {})
 
         managed_prefix_list = self.ec2_backend.create_managed_prefix_list(
             address_family=address_family,
@@ -282,7 +279,7 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(CREATE_MANAGED_PREFIX_LIST)
         return template.render(managed_prefix_list=managed_prefix_list)
 
-    def describe_managed_prefix_lists(self):
+    def describe_managed_prefix_lists(self) -> str:
         prefix_list_ids = self._get_multi_param("PrefixListId")
         filters = self._filters_from_querystring()
         managed_prefix_lists = self.ec2_backend.describe_managed_prefix_lists(
@@ -291,7 +288,7 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(DESCRIBE_MANAGED_PREFIX_LIST)
         return template.render(managed_prefix_lists=managed_prefix_lists)
 
-    def get_managed_prefix_list_entries(self):
+    def get_managed_prefix_list_entries(self) -> str:
         prefix_list_id = self._get_param("PrefixListId")
         target_version = self._get_param("TargetVersion")
         managed_prefix_list = self.ec2_backend.get_managed_prefix_list_entries(
@@ -310,7 +307,7 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(GET_MANAGED_PREFIX_LIST_ENTRIES)
         return template.render(entries=entries)
 
-    def delete_managed_prefix_list(self):
+    def delete_managed_prefix_list(self) -> str:
         prefix_list_id = self._get_param("PrefixListId")
         managed_prefix_list = self.ec2_backend.delete_managed_prefix_list(
             prefix_list_id
@@ -318,7 +315,7 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(DELETE_MANAGED_PREFIX_LIST)
         return template.render(managed_prefix_list=managed_prefix_list)
 
-    def describe_prefix_lists(self):
+    def describe_prefix_lists(self) -> str:
         prefix_list_ids = self._get_multi_param("PrefixListId")
         filters = self._filters_from_querystring()
         managed_pls = self.ec2_backend.describe_managed_prefix_lists(
@@ -327,7 +324,7 @@ class VPCs(EC2BaseResponse):
         template = self.response_template(DESCRIBE_PREFIX_LIST)
         return template.render(managed_pls=managed_pls)
 
-    def modify_managed_prefix_list(self):
+    def modify_managed_prefix_list(self) -> str:
         add_entry = self._get_multi_param("AddEntry")
         prefix_list_id = self._get_param("PrefixListId")
         current_version = self._get_param("CurrentVersion")
@@ -378,7 +375,7 @@ CREATE_VPC_RESPONSE = """
               {% endfor %}
           </ipv6CidrBlockAssociationSet>
         {% endif %}
-      <dhcpOptionsId>{% if vpc.dhcp_options %}{{ vpc.dhcp_options.id }}{% else %}dopt-1a2b3c4d2{% endif %}</dhcpOptionsId>
+      <dhcpOptionsId>{% if vpc.dhcp_options %}{{ vpc.dhcp_options.id }}{% else %}default{% endif %}</dhcpOptionsId>
       <instanceTenancy>{{ vpc.instance_tenancy }}</instanceTenancy>
       <ownerId> {{ vpc.owner_id }}</ownerId>
       <tagSet>
@@ -479,7 +476,7 @@ DESCRIBE_VPCS_RESPONSE = """
               {% endfor %}
             </ipv6CidrBlockAssociationSet>
         {% endif %}
-        <dhcpOptionsId>{% if vpc.dhcp_options %}{{ vpc.dhcp_options.id }}{% else %}dopt-7a8b9c2d{% endif %}</dhcpOptionsId>
+        <dhcpOptionsId>{% if vpc.dhcp_options %}{{ vpc.dhcp_options.id }}{% else %}default{% endif %}</dhcpOptionsId>
         <instanceTenancy>{{ vpc.instance_tenancy }}</instanceTenancy>
         <isDefault>{{ vpc.is_default }}</isDefault>
         <ownerId> {{ vpc.owner_id }}</ownerId>
@@ -764,7 +761,7 @@ DESCRIBE_VPC_ENDPOINT_RESPONSE = """<DescribeVpcEndpointsResponse xmlns="http://
 
 DELETE_VPC_ENDPOINT_RESPONSE = """<DeleteVpcEndpointsResponse xmlns="http://ec2.amazonaws.com/doc/2016-11-15/">
     <requestId>19a9ff46-7df6-49b8-9726-3df27527089d</requestId>
-    <unsuccessful>{{ 'Error' if not response else '' }}</unsuccessful>
+    <unsuccessful></unsuccessful>
 </DeleteVpcEndpointsResponse>"""
 
 
